
/**
 * Title:        Appia<p>
 * Description:  Protocol development and composition framework<p>
 * Copyright:    Copyright (c) Alexandre Pinto & Hugo Miranda & Luis Rodrigues<p>
 * Company:      F.C.U.L.<p>
 * @author Alexandre Pinto & Hugo Miranda & Luis Rodrigues
 * @version 1.0
 */
package appia;

public class ChannelCursor {

  Session[] sessions;
  Layer[] layers;
  Channel channel;
  int pos=-1;

  public ChannelCursor(Channel channel) {
    this.channel=channel;
    this.sessions=channel.sessions;
    this.layers=channel.getQoS().layers;
  }

  public Session down() throws AppiaCursorException {
    if (pos < 0)
       throw new AppiaCursorException(AppiaCursorException.CURSORNOTSET);

    if (pos <= 0)
       throw new AppiaCursorException(AppiaCursorException.CURSORONBOTTOM);

    return sessions[--pos];
  }

  public Session up() throws AppiaCursorException {
    if (pos < 0)
       throw new AppiaCursorException(AppiaCursorException.CURSORNOTSET);

    if (pos >= (sessions.length-1))
       throw new AppiaCursorException(AppiaCursorException.CURSORONTOP);

    return sessions[++pos];
  }

  public Session top() {
    pos=sessions.length-1;
    return sessions[pos];
  }

  public Session bottom() {
    pos=0;
    return sessions[pos];
  }

  public boolean isPositioned() {
    return (pos >= 0) && (pos < sessions.length);
  }

  public void setSession(Session session) throws AppiaCursorException {
    if (pos < 0)
       throw new AppiaCursorException(AppiaCursorException.CURSORNOTSET);

    if (sessions[pos] != null)
       throw new AppiaCursorException(AppiaCursorException.ALREADYSET,"Session has already been Set");

    if (layers[pos] != session.getLayer())
       throw new AppiaCursorException(AppiaCursorException.WRONGLAYER,"Wrong Session Layer");

    sessions[pos]=session;
  }

  public Layer getLayer() throws AppiaCursorException {
    if (pos<0)
       throw new AppiaCursorException(AppiaCursorException.CURSORNOTSET);

    return layers[pos];
  }
}