
/**
 * Title:        Appia<p>
 * Description:  Protocol development and composition framework<p>
 * Copyright:    Copyright (c) Alexandre Pinto & Hugo Miranda & Luis Rodrigues<p>
 * Company:      F.C.U.L.<p>
 * @author Alexandre Pinto & Hugo Miranda & Luis Rodrigues
 * @version 1.0
 */
package appia;

import java.io.PrintStream;
import appia.events.channel.ChannelEvent;

public class Event {

  private int currentSession=-1;
  private int firstSession=-1;

	private Channel channel;
	private EventScheduler eventScheduler;
	private Direction direction;
	private Session src;

  private Session[] route;
  private boolean channelEvent;

  private boolean isInitiated=false;

	public Event() {}

	public Event(Channel channel, Direction direction, Session src)
    throws AppiaEventException {

		this.channel=channel;
		this.direction=direction;
		this.src=src;

    init();
	}

	public void setDirection(Direction dir) {
		this.direction=dir;
    isInitiated=false;
	}

	public Direction getDirection() {
		return direction;
	}

	public void setChannel(Channel channel) {
		this.channel=channel;
    isInitiated=false;
	}

	public Channel getChannel() {
		return channel;
	}

	public void setSource(Session src) {
		this.src=src;
    isInitiated=false;
	}

	public Session getSource() {
		return src;
	}

  public final boolean isAccepted() {
    return true;
  }

	public final Session popSession() {

    // first session
		if (currentSession < 0)
       currentSession=firstSession;
		else {
      //if event direction is down and there are no more sessions, then next is
      // the channel itself
			if ((currentSession==0) && (direction.direction==Direction.DOWN))
         currentSession=route.length;
			else {
        //if is still in the route continue
        if ( currentSession < route.length )
           currentSession+=direction.direction;
        else {
          //if is in the channel itself
          if (currentSession == route.length)
             currentSession=Integer.MAX_VALUE;
        }
      }
		}

    if ( (currentSession >= 0) && (currentSession < route.length) )
       return route[currentSession];
    else {
      if ( (currentSession == route.length) && channelEvent )
         channel.handle(this);

      return null;
    }
	}

  public Session currentSession() {
    if ( (currentSession >= 0) && (currentSession < route.length) )
       return route[currentSession];
    else
      return null;
  }

	public final void go() throws AppiaEventException {
		if (!isInitiated)
			 throw new AppiaEventException(AppiaEventException.NOTINITIALIZED,"Event not initialized");

		eventScheduler.insert(this);
	}

	public final void init() throws AppiaEventException {
		if (channel==null)
			throw new AppiaEventException(AppiaEventException.ATTRIBUTEMISSING,"Missing Event attribute: Channel");

		if (direction==null)
			throw new AppiaEventException(AppiaEventException.ATTRIBUTEMISSING,"Missing Event attribute: Direction");

    ChannelEventRoute channelRoute=channel.getEventRoute(this);
    firstSession=channel.getFirstSession(channelRoute,direction,src);
    currentSession=-1;

    route=channelRoute.getRoute();
    channelEvent=(this instanceof ChannelEvent);

    eventScheduler=channel.getEventScheduler();

    isInitiated=true;
	}

  public void debug(PrintStream out) {
    out.println("Event:");
    out.println(this);
  }

  public Event cloneEvent() throws CloneNotSupportedException {
    Event e=(Event)clone();

    //deep copy
    e.direction=new Direction(direction.direction);
    e.firstSession=-1;
    e.currentSession=-1;
    e.isInitiated=false;

    return e;
  }
}