
/**
 * Title:        Appia<p>
 * Description:  Protocol development and composition framework<p>
 * Copyright:    Copyright (c) Alexandre Pinto & Hugo Miranda & Luis Rodrigues<p>
 * Company:      F.C.U.L.<p>
 * @author Alexandre Pinto & Hugo Miranda & Luis Rodrigues
 * @version 1.0
 */
package appia;


public class EventScheduler {

	private class Entry {
    public Event event;
    public Entry next;

    public Entry(Entry next) {
      this.next=next;
    }
  }


  private class EntryPool {

    public static final long MIN=100;

    Entry free=null;
    long used=0;
    long total=0;
    long min_threshold;

    public EntryPool() {
      increase();
    }

    public void increase() {
      long i;
      long newtotal= (total == 0) ? MIN : total*2 ;

      for(i=total ; i < newtotal ; i++) {
        free=new Entry(free);
      }

      total=newtotal;
      min_threshold= (total <= MIN) ? (-1) : (total/4) ;
    }

    public void decrease() {
      if (total > MIN) {
        long i;
        long newtotal=total/2;

        for(i=total ; i > newtotal ; i--) {
          free=free.next;
        }

        total=newtotal;
        min_threshold= (total <= MIN) ? (-1) : (total/4) ;
      }
    }

    public Entry get() {
      Entry e=free;
      free=free.next;
      used++;

      if (used == total)
         increase();

      return e;
    }

    public void put(Entry e) {
      e.next=free;
      free=e;
      used--;

      if (used == min_threshold)
         decrease();
    }
  }

  private EntryPool pool=new EntryPool();

  private Entry head=null;
  private Entry tail=null;
  private Entry last=null;

  private int currentDirection=0;
  private Channel currentChannel=null;
  private Session currentSession=null;

  int nEvents=0;

	public synchronized void insert(Event event) {
    Entry e=pool.get();
    e.event=event;

    if (
       (currentSession != null) &&
       (event.getDirection().direction == currentDirection) &&
       (event.getChannel() == currentChannel) &&
       ((event.currentSession() == currentSession) || (event.getSource() == currentSession))
       ) {

      if (head == null) {
         e.next=null;
         head=tail=e;
      } else {

        if (tail == last)
           tail=e;

        if (last == null) {
          e.next=head;
          head=e;
        } else {
          e.next=last.next;
          last.next=e;
        }
      }

      last=e;

    } else {
      e.next=null;

      if (head == null)
         head=tail=e;
      else {
        tail.next=e;
        tail=e;
      }
    }

    nEvents++;
    Appia.insertedEvent();
	}

	public synchronized boolean consumeEvent() {
    if (nEvents <= 0)
       return false;

    Entry e=head;
    Event ev=e.event;

    if (head == tail)
       tail=null;
    head=head.next;
    last=null;
    pool.put(e);

    currentSession=ev.popSession();

    if (currentSession != null) {

      currentDirection=ev.getDirection().direction;
      currentChannel=ev.getChannel();

      currentSession.handle(ev);

      currentSession=null;
    }

    nEvents--;
    return true;
	}

	public void start() {}
	public void stop() {}

}