
/**
 * Title:        Appia<p>
 * Description:  Protocol development and composition framework<p>
 * Copyright:    Copyright (c) Alexandre Pinto & Hugo Miranda & Luis Rodrigues<p>
 * Company:      F.C.U.L.<p>
 * @author Alexandre Pinto & Hugo Miranda & Luis Rodrigues
 * @version 1.0
 */
package appia;

import java.util.HashSet;
import java.util.Iterator;

public class QoS {

  private String ID;

  protected QoSEventRoute[] eventsRoutes;
  protected Layer[] layers;

  protected Class[][] eventsAccepted=null;
  private HashSet eventsProvided=new HashSet();

  public QoS(String ID, Layer[] layers) throws AppiaInvalidQoSException {
    this.layers=layers;
    this.ID=ID;

    eventsAccepted=new Class[layers.length+1][];

    int i,j;
    //gathers provided and accepted events from layers
    for (i=0 ; i < eventsAccepted.length-1 ; i++) {
      Class[] provides=layers[i].getProvidedEvents();

      for (j=0 ; j<provides.length ; j++) {
        eventsProvided.add(provides[j]);
      }

      eventsAccepted[i]=layers[i].getAcceptedEvents();
    }

    //gathers provided and accepted events of the channel
    try {
      eventsProvided.add(Class.forName("appia.events.channel.ChannelInit"));
      eventsProvided.add(Class.forName("appia.events.channel.ChannelClose"));
      eventsProvided.add(Class.forName("appia.events.channel.AsyncEvent"));
      eventsAccepted[eventsAccepted.length-1]=new Class[1];
      eventsAccepted[eventsAccepted.length-1][0]=Class.forName("appia.events.channel.ChannelEvent");
    } catch (ClassNotFoundException e) {
      throw new AppiaError("Some \"appia\" Event Classes have not been found:"+e.getMessage());
    }

    eventsRoutes=new QoSEventRoute[eventsProvided.size()];

    validateQoS();
    this.layers=(Layer[])layers.clone();
    makeEventsRoutes();
  }

  public String getQoSID() {
    return ID;
  }

  public Channel createUnboundChannel(String channelID, EventScheduler eventScheduler) {
    return new Channel(channelID, this, eventScheduler);
  }

  public Channel createUnboundChannel(String channelID) {
    return new Channel(channelID, this, new EventScheduler());
  }

  public Layer[] getLayers() {
    return (Layer[])layers.clone();
  }

  protected void validateQoS() throws AppiaInvalidQoSException {
    int i,j;

    //checks if all required events belong to provided
    for (i=0 ; i < layers.length ; i++) {
      Class[] requires=layers[i].getRequiredEvents();

      for (j=0 ; (requires != null) && (j < requires.length) ; j++) {
        if ( ! hasRequired(requires[j]) ) {
          throw new AppiaInvalidQoSException(" required Event (\""+requires[j].getName()+"\") not provided");
        }
      }
    }
  }

  protected void makeEventsRoutes() {
    //creates QoSEventRoute for all provided events
    Iterator provides=eventsProvided.iterator();

    for (int k=0 ; provides.hasNext() ; k++) {
      Class eventType=(Class)provides.next();

      eventsRoutes[k]=new QoSEventRoute(this,eventType);
    }
  }

  public QoSEventRoute[] getEventsRoutes() {
    return eventsRoutes;
  }

  private boolean hasRequired(Class required) {
    boolean result=false;
    Iterator provides=eventsProvided.iterator();

    while (provides.hasNext() && !result) {
      result= required.isAssignableFrom((Class)provides.next());
    }

    return result;
  }

}