
package appia.events.channel;

import appia.*;

/**
 * Event signifying an <i>asynchronous</i> ocorrence.
 * <br>
 * This event should be requested, generated, using the
 * {@link appia.Channel#makeAsync Channel.makeAsync()}
 * method.
 * <br>
 * It is usefull so that {@link appia.Session sessions} can respond to
 * asynchronous events, generated outside the
 * {@link appia.Channel Channel}. Thus, it eliminates the use of
 * {@link appia.events.channel.PeriodicTimer periodic timers}
 * to respond to such events.
 *
 * @author Alexandre Pinto
 * @version 0.1
 * @see appia.Channel#makeAsync
 */
public final class AsyncEvent extends ChannelEvent {

  /**
   * Creates an <b>initiated</b> <i>AsyncEvent</i>.
   * <br>
   * An <i>AsyncEvent</i> should not be explicitly created.
   * When such event is required the
   * {@link appia.Channel#makeAsync Channel.makeAsync()}
   * method should be called instead.
   *
   * @see appia.Channel#makeAsync
   * @param channel the {@link appia.Channel Channel} where the event will be
   *                put.
   * @throws appiaEventException exception thrown if <i>channel</i> is null.
   */
  public AsyncEvent(Channel channel) throws AppiaEventException {
    super(
      channel,
      new Direction(Direction.UP),
      null,
      new EventQualifier(EventQualifier.NOTIFY)
    );
  }
}