
/**
 * Title:        <p>
 * Description:  <p>
 * Copyright:    Copyright (c) <p>
 * Company:      <p>
 * @author
 * @version 1.0
 */
package appia.message;

public class Message {

  public static final int INCREASE=1000;

  public class Block {
    public byte[] buf;
    public int offset;
    public int length;
    public int off;
    public int len;
    public Block next=null;

    public Block() {}

    public Block(byte[] buf, int off, int len, int pos) {
      this.buf=buf;
      this.offset=off;
      this.length=len;
      this.off=pos;
      this.len=length-(pos-off);
    }

    public Block myClone() {
      Block b=new Block();
      b.buf=(byte[])buf.clone();
      b.offset=offset;
      b.length=length;
      b.off=off;
      b.len=len;
      b.next=next;
      return b;
    }
  }

  protected Block first=null;
  protected int size = 0;

  public Message() {}

  /*
  public Message(byte[] data, int offset, int length) {
    Block b=new Block(data,offset,length,offset);
    first=b;
    size=length;
  }
  */

  public void setByteArray(byte[] data, int offset, int length) {
    Block b=new Block(data,offset,length,offset);
    first=b;
    size=length;
  }

  public int length() {
    return size;
  }

  public void peek(MsgBuffer mbuf) {
    int l=Math.min(first.len,mbuf.len);
    mbuf.data=first.buf;
    mbuf.off=first.off;
    mbuf.len=l;
  }

  public int discard(int length) {
    int remain=length;
    Block b;

    for ( b=first ; (remain > 0) && (b != null) ; b=b.next ) {
      if (remain >= b.len) {
        remain -= b.len;
        first=b.next;
      } else {
        b.off+=remain;
        b.len-=remain;
        remain=0;
      }
    }

    size=Math.max(size-length,0);
    return length-remain;
  }

  public void pop(MsgBuffer mbuf) {
    int l=Math.min(first.len,mbuf.len);

    mbuf.data=first.buf;
    mbuf.off=first.off;
    mbuf.len=l;

    first.off+=l;
    first.len-=l;

    if (first.len == 0)
       first=first.next;

    size-=l;
  }

  public void push(MsgBuffer mbuf) {
    int l=mbuf.len;

    if ( (first == null) || (l > (first.off - first.offset)) ) {
      byte[] a=new byte[l+INCREASE];
      Block b=new Block(a,0,a.length,a.length-l);

      b.next=first;
      first=b;
    } else {
      first.off-=l;
      first.len+=l;
    }

    mbuf.data=first.buf;
    mbuf.off=first.off;

    size+=l;
  }

  public int truncate(int newLength) {
    int remain=newLength;
    Block b;

    for ( b=first ; (remain > 0) && (b != null) ; b=b.next ) {
      if (b.len >= remain) {
        b.len=remain;
        b.next=null;
        remain=0;
        //the following bytes are removed by Garbage Collection
      } else {
        remain-=b.len;
      }
    }

    size=Math.min(newLength,size);
    return newLength-remain;
  }

  public void frag(Message m, int length) {
    int remain=length;
    Block b;

    for ( b=first ; (remain > 0) && (b != null) ; b=b.next ) {
      if (b.len == remain) {
        m.first=b.next;
        b.next=null;
      } else {
        if (b.len > remain) {
          m.first=b.myClone();
          m.first.off+=remain;
          m.first.len-=remain;

          b.next=null;
          b.len=remain;
        } else {
          remain-=b.len;
        }
      }
    }

    m.size=Math.max(size-length,0);
    size=Math.min(size,length);
  }

  public void join(Message m) {
    if (first == null) {
      first=m.first;
    } else {
      Block b;

      for ( b=first ; b.next != null ; b=b.next ) ;

      b.next=m.first;
    }

    size+=m.size;
  }

  public byte[] toByteArray() {
    int len=length();

    byte[] array=new byte[len];
    int off=0;

    Block b;
    for ( b=first ; b != null ; b=b.next ) {
      System.arraycopy(b.buf,b.off,array,off,b.len);
      off+=b.len;
    }

    return array;
  }

  public MsgWalk getMsgWalk() {
    return new MsgWalk(first);
  }
}