package appia.protocols.drop;

//////////////////////////////////////////////////////////////////////
//                                                                  //
// Appia: protocol development and composition framework            //
//                                                                  //
// Version: 1.0/J                                                   //
//                                                                  //
// Copyright, 2000, Universidade de Lisboa                          //
// All rights reserved                                              //
// See license.txt for further information                          //
//                                                                  //
// Class: DropSession: Randomly drop sending messages               //
//                                                                  //
// Author: Hugo Miranda, 05/2000                                    //
//                                                                  //
// Change Log:                                                      //
// Data, alteracao, razao/consequncias                             //
//                                                                  //
//////////////////////////////////////////////////////////////////////

import appia.*;
import appia.events.*;

/**
 * Class DropSession randomly drops outgoing messages. It should be
 * placed between socket sending session and reliable delivery

 * @author Hugo Miranda
 * @see    DropLayer
 * @see    Session
 */

public class DropSession extends Session {

    /**
     * Percentage of messages to be dropped. This is a public static
     * value that can be changed any time. 0 prevents messages from
     * being lost.
     */

    public static double dropRate=0.5;

    /**
     * Mainly used for corresponding layer initialization
     */

    public DropSession(DropLayer l) {
	super(l);
    }

    /**
     * This protocol does not attempt to bind any sessions on channels
     */

    public void boundSessions(Channel channel) {
    }

    /**
     * Main Event handler function. Accepts all incoming events and
     * dispatches them to the appropriate functions. Only SendableEvents
     * are expected.
     * @ params e The incoming event
     * @ see Session
     */

    public void handle(Event e) {
	if(e.getDirection().direction==Direction.UP ||
	   Math.random()>=dropRate) {
	    try {
		e.go();
		//		System.err.println("drop: passed");
	    }
	    catch(AppiaEventException ex) {
		System.err.println("Unexpected exception in Drop Session");
	    }
	}
	/*	else
	    System.err.println("drop: message dropped");
	*/    }
	
    public Layer getLayer() {
	return layer;
    }
}
