package appia.protocols.ghost;

//////////////////////////////////////////////////////////////////////
//                                                                  //
// Appia: protocol development and composition framework            //
//                                                                  //
// Version: 1.0/J                                                   //
//                                                                  //
// Copyright, 2000, Universidade de Lisboa                          //
// All rights reserved                                              //
// See license.txt for further information                          //
//                                                                  //
// Class: HASession: Handle All Session              //
//                                                                  //
// Author: Hugo Miranda, 05/2000                                    //
//                                                                  //
// Change Log:                                                      //
// Data, alteracao, razao/consequncias                             //
//                                                                  //
//////////////////////////////////////////////////////////////////////

import appia.*;

/**
 * Class GhostSession accepts every event
 * flowing on the channel and forwards
 * it immediately. The goal is to mesure
 * latency avoidance by bypassing layers
 * not interested in events.
 *
 * @author Hugo Miranda
 * @see    HALayer
 * @see    Session
 */

public class GhostSession extends Session {

    /**
     * Mainly used for corresponding layer initialization
     */

    public GhostSession(Layer l) {
	super(l);
    }

    /**
     * This protocol does not attempt to bind any sessions on channels
     */

    public void boundSessions(Channel channel) {
    }

    /**
     * Main Event handler function. Accepts all incoming events and
     * forwards them
     * @ params e The incoming event
     * @ see Session
     */

    public void handle(Event e) {
	try {
	    e.go();
	}
	catch(AppiaEventException ex) {}
    }
	
    public Layer getLayer() {
	return layer;
    }
}
