
/**
 * Title:        Apia<p>
 * Description:  Protocol development and composition framework<p>
 * Copyright:    Copyright (c) Alexandre Pinto & Hugo Miranda & Luis Rodrigues<p>
 * Company:      F.C.U.L.<p>
 * @author Alexandre Pinto & Hugo Miranda & Luis Rodrigues
 * @version 1.0
 */
package appia.protocols.group;

import appia.*;
import appia.protocols.group.events.*;
import appia.protocols.udpsimple.InetWithPort;
import appia.message.*;

import java.util.Vector;

import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;

import appia.events.channel.Debug;
import java.io.PrintStream;

public class GroupBottomSession extends Session {

  public GroupBottomSession(Layer layer) {
    super(layer);
  }

  public void handle(Event event) {

    // GroupSendableEvent
    if (event instanceof GroupSendableEvent) {
      // am i intialized
      if (vs == null) {
        debug("event (\""+event.getClass().getName()+"\") discarded because i haven't received a view");
        return;
      }

      // BlockOk
      if (event instanceof BlockOk)
         blocked=true;

      if (event.getDirection().direction == Direction.DOWN)
         handleDownGroupSendableEvent((GroupSendableEvent)event);
      else
        handleUpGroupSendableEvent((GroupSendableEvent)event);
      return;
    }

    // View
    if (event instanceof View) {
      handleView((View)event);
      return;
    }

    // GroupInit
    if (event instanceof GroupInit) {
      ip_multicast=((GroupInit)event).ip_multicast;
      try { event.go(); } catch (AppiaEventException ex) { ex.printStackTrace(); }
      return;
    }

    // Debug
    if (event instanceof Debug) {
      Debug ev=(Debug)event;

      if (ev.getQualifier().isOn()) {
        if (ev.getOutput() instanceof PrintStream)
           debug=(PrintStream)ev.getOutput();
        else
          debug=new PrintStream(ev.getOutput());
      } else {
        if (ev.getQualifier().isOff())
           debug=null;
      }
    }

    debug("discarded unwanted event \""+event.getClass().getName()+"\"");
  }

  private ViewState vs;
  private LocalState ls;
  private InetWithPort ip_multicast;
  private int[] all;
  private boolean blocked;
  private Vector buffer=new Vector();

  private void handleUpGroupSendableEvent(GroupSendableEvent ev) {
    try {
      deserialize(ev,ev.getMessage());
    } catch (IOException ex) {
      ex.printStackTrace();
      System.err.println("appia:group:GroupBottomSession: event discarded due to impossibility to deserialize message");
      return;
    }

    if (vs == null) {
      buffer.add(ev);
      return;
    }

    if (blocked && vs.group.equals(ev.group) && !vs.id.equals(ev.view_id)) {
      buffer.add(ev);
      return;
    }

    deliver(ev);
  }

  private void handleDownGroupSendableEvent(GroupSendableEvent ev) {
    try {
      serialize(ev,ev.getMessage());
    } catch (IOException ex) {
      ex.printStackTrace();
      System.err.println("appia:group:GroupBottomSession: event discarded due to impossibility to serialize message");
      return;
    }

    ev.source=vs.addresses[ls.my_rank];

    if (ev instanceof Send) {
      send((int[])ev.dest,ev);
    } else { // Cast
      if (ip_multicast == null) {
        send(all,ev);
      } else { // multicast
        ev.dest=ip_multicast;
        try {
          ev.go();
        } catch (AppiaEventException ex) {
          ex.printStackTrace();
          System.err.println("appia:group:GroupBottomSession: event discarded");
        }
      }
    }
  }

  private void handleView(View ev) {
    int i;

    vs=ev.vs;
    ls=ev.ls;

    all=new int[vs.view.length];
    for(i=0 ; i < all.length ; i++) all[i]=i;

    try { ev.go(); } catch (AppiaEventException ex) { ex.printStackTrace(); }

    for (i=0 ; i < buffer.size() ; i++) {
      deliver((GroupSendableEvent)buffer.get(i));
    }
    buffer.clear();
    blocked=false;
  }

  private void send(int[] dests, GroupSendableEvent ev) {
    int i;

    for(i=0 ; i < dests.length-1 ; i++) {
      if (dests[i] != ls.my_rank) {
        try {
          GroupSendableEvent clone=(GroupSendableEvent)ev.cloneEvent();
          clone.dest=vs.addresses[dests[i]];
          clone.setSource(this);
          clone.init();
          clone.go();
        } catch (CloneNotSupportedException ex) {
          throw new AppiaGroupError("GroupBottomSession: "+ex.getMessage());
        } catch (AppiaEventException ex) {
          throw new AppiaGroupError("GroupBottomSession: "+ex.getMessage()+"(possible violation of \"all or nothing\" property");
        }
      }
    }

    //TODO: erase. for testing porpuses only
    //if (ev instanceof appia.test.TestGSEvent)
    //   return;

    if ( (dests.length > 0) && (dests[dests.length-1] != ls.my_rank) ) {
      ev.dest=vs.addresses[dests[dests.length-1]];
      try {
        ev.go();
      } catch (AppiaEventException ex) {
        throw new AppiaGroupError("GroupBottomSession: "+ex.getMessage()+"(possible violation of \"all or nothing\" property");
      }
    }
  }

  private void deliver(GroupSendableEvent ev) {

    if (!vs.group.equals(ev.group) || !vs.id.equals(ev.view_id) ) {
      debug("discarded event ("+ev.getClass().getName()+") due to bad \"group\" and/or \"view id\"");
      return;
    }

    ev.dest=null;

    if ( (ev.source != null) && ((ev.orig=vs.getRankByAddress((InetWithPort)ev.source)) >= 0) ) {
      ev.source=vs.view[ev.orig];
      try {
        ev.go();
      } catch (AppiaEventException ex) {
        ex.printStackTrace();
        System.err.println("appia:group:GroupBottomSession: up event discarded");
      }
    } else {
      System.err.println("appia:group:GroupBottomSession: event discarded due to bad source address");
    }
  }

  private AppiaOutputStream oStream= new AppiaOutputStream();
  private AppiaInputStream iStream= new AppiaInputStream();

  private void serialize(GroupSendableEvent ev, Message msg) throws IOException {
    ObjectsMessage omsg=ev.getObjectsMessage();

    omsg.push(ev.view_id);
    omsg.push(ev.group);

    omsg.flush();

  //oStream.writeObjectToMessage(omsg,msg);

  /*
    ObjectOutputStream o=new ObjectOutputStream(oStream);

    o.writeObject(ev.group);
    o.writeObject(ev.view_id);
    o.close();

    MsgBuffer mbuf=new MsgBuffer();
    mbuf.len=oStream.size();
    msg.push(mbuf);

    oStream.writeToMsgBuffer(mbuf);
    oStream.reset();
  */

    //System.out.println("s msg size="+msg.length()+"\""+msg.toByteArray().toString()+"\"");

  }

  private void deserialize(GroupSendableEvent ev, Message msg) throws IOException {

    //System.out.println("d msg size="+msg.length()+"\""+msg.toByteArray().toString()+"\"");

  /*
    ObjectsMessage omsg=null;

    try {
      omsg=(ObjectsMessage)iStream.readObjectFromMessage(msg);
    } catch (ClassNotFoundException ex) {
      throw new IOException("ClassNotFound:"+ex.getMessage());
    }
  */

    ObjectsMessage omsg=ev.getObjectsMessage();

    ev.group=(Group)omsg.pop();
    ev.view_id=(ViewID)omsg.pop();

    if ( (ev.group == null) || (ev.view_id == null) )
       throw new IOException("!!!!!"+ev.getClass().getName()+"!!!!!!!!");

    //ev.setObjectsMessage(omsg);

  /*
    MsgBuffer mbuf=new MsgBuffer();
    mbuf.len=msg.length();
    msg.peek(mbuf);

    iStream.importMsgBuffer(mbuf);

    ObjectInputStream i=new ObjectInputStream(iStream);

    try {
      ev.group=(Group)i.readObject();
      ev.view_id=(ViewID)i.readObject();
    } catch (ClassNotFoundException ex) {
      throw new IOException("ClassNotFoundException");
    }

    msg.discard(iStream.bytesRead());
  */
  }

  private PrintStream debug=null;

  private void debug(String s) {
    if (debug != null)
       debug.println("appia:group:GroupBottomSession: "+s);
  }

}