
/**
 * Title:        Apia<p>
 * Description:  Protocol development and composition framework<p>
 * Copyright:    Copyright (c) Alexandre Pinto & Hugo Miranda & Luis Rodrigues<p>
 * Company:      F.C.U.L.<p>
 * @author Alexandre Pinto & Hugo Miranda & Luis Rodrigues
 * @version 1.0
 */
package appia.protocols.group;

import appia.*;

public class SuspectLayer extends Layer {

  public static final long SUSPECT_SWEEP=1000; //in milliseconds
  public static final int SUSPECT_IDLE=5;     //in number of sweeps

  public SuspectLayer() {
    try {
      Class alive=Class.forName("appia.protocols.group.events.Alive");
      Class view=Class.forName("appia.protocols.group.events.View");
      Class suspect=Class.forName("appia.protocols.group.events.Suspect");
      Class fail=Class.forName("appia.protocols.group.events.Fail");
      Class periodic=Class.forName("appia.protocols.group.events.SuspectTimer");
      Class echo=Class.forName("appia.events.channel.EchoEvent");
      Class undeilvered=Class.forName("appia.protocols.fifo.FIFOUndelivered");

      evProvide=new Class[5];
      evProvide[0]=alive;
      evProvide[1]=suspect;
      evProvide[2]=fail;
      evProvide[3]=periodic;
      evProvide[4]=echo;

      evRequire=new Class[1];
      evRequire[0]=view;

      evAccept=new Class[4];
      evAccept[0]=alive;
      evAccept[1]=suspect;
      evAccept[2]=view;
      evAccept[3]=periodic;
    } catch (ClassNotFoundException ex) {
      throw new AppiaGroupError("SuspectLayer:"+ex.getMessage());
    }
  }

  public Session createSession() {
    return new SuspectSession(this,SUSPECT_SWEEP,SUSPECT_IDLE);
  }
}