
/**
 * Title:        Apia<p>
 * Description:  Protocol development and composition framework<p>
 * Copyright:    Copyright (c) Alexandre Pinto & Hugo Miranda & Luis Rodrigues<p>
 * Company:      F.C.U.L.<p>
 * @author Alexandre Pinto & Hugo Miranda & Luis Rodrigues
 * @version 1.0
 */
package appia.protocols.group;

import appia.protocols.udpsimple.InetWithPort;

public class ViewState implements java.io.Serializable {

  public String version;
  public Group group;
  public ViewID id;
  public ViewID[] previous;
  public Endpt[] view;
  public InetWithPort[] addresses;

  public int getRank(Endpt endpt) {
    int i;
    for(i=view.length-1 ; (i >= 0) && !endpt.equals(view[i]) ; i--);
    return i;
  }

  public int getRankByAddress(InetWithPort address) {
    int i;
    for(i=addresses.length-1 ; (i >= 0) && !address.equals(addresses[i]) ; i--);
    return i;
  }

  public ViewState(
          String version,
          Group group,
          ViewID id,
          ViewID[] previous,
          Endpt[] view,
          InetWithPort[] addresses) throws NullPointerException,AppiaGroupException {

    if ( (group==null) || (id==null) || (view==null) || (addresses==null) )
       throw new NullPointerException("appia:group:ViewState: group or view_id or view or addresses");

    if ( view.length != addresses.length )
       throw new AppiaGroupException("ViewState: view.length != addresses.length");

    this.version=version;
    this.group=group;
    this.id=id;
    this.previous=previous;
    this.view=view;
    this.addresses=addresses;
  }

  public String toString() {
    String s="";
    int i;

    s="\nversion: "+version+"\ngroup: "+group.toString()+"\nid: "+id.toString();
    s=s+"\nprevious: [";
    for (i=0 ; i < previous.length ; i++) s=s+previous[i].toString()+",";
    s=s+"]\nview: [";
    for (i=0 ; i < view.length ; i++) s=s+view[i].toString()+",";
    s=s+"]\naddresses: [";
    for (i=0 ; i < addresses.length ; i++)
        s=s+"("+addresses[i].host.toString()+","+addresses[i].port+"),";
    s=s+"]\n";

    return s;
  }
}
