package appia.protocols.udpsimple;

//////////////////////////////////////////////////////////////////////
//                                                                  //
// Appia: protocol development and composition framework            //
//                                                                  //
// Version: 1.0/J                                                   //
//                                                                  //
// Copyright, 2000, Universidade de Lisboa                          //
// All rights reserved                                              //
// See license.txt for further information                          //
//                                                                  //
// Class: UdpSimpleLayer: Unreliable send/receive using UDP/IP      //
//                                                                  //
// Author: Hugo Miranda, 05/2000                                    //
//                                                                  //
// Change Log:                                                      //
// Data, alteracao, razao/consequncias                             //
//                                                                  //
//////////////////////////////////////////////////////////////////////

import appia.*;
import appia.events.*;

/**
 * Class UdpSimpleLayer is the Layer subclassing for UdpSimple protocol.
 *
 * @see appia.Layer
 * @author Hugo Miranda
 */

public class UdpSimpleLayerExt extends Layer {

    public Session createSession() {
	return new UdpSimpleSession(this);
    }

    public Class[] getProvidedEvents() {
	Class[] ret=new Class[1];
	try {
	    ret[0]=Class.forName("appia.events.SendableEvent");
	}
	catch(ClassNotFoundException ex) {
	    System.err.println("Class Not Found Exception in "+
			       "UdpSimpleLayer (getProvidedEvents). "+
			       "Check CLASSPATH information.");
	}

	return ret;
    }

    public Class[] getRequiredEvents() {
	Class[] ret=new Class[1];
	try {
	    ret[0]=Class.forName("appia.protocols.udpsimple.RegisterSocketEvent");
	}
	catch(ClassNotFoundException ex) {
	    System.err.println("Class Not Found Exception in "+
			       "UdpSimpleLayer (getRequiredEvents). "+
			       "Check CLASSPATH information.");
	}

	return ret;
    }
    
    public Class[] getAcceptedEvents() {
	Class[] ret=new Class[1];
	try {
	    ret[0]=Class.forName("appia.Event");
	}
	catch(ClassNotFoundException ex) {
	    System.err.println("Class Not Found Exception in "+
			       "UdpSimpleLayer (getAcceptedEvents). "+
			       "Check CLASSPATH information.");
	}
	return ret;
    }

    public void channelDispose(Session s,Channel c) {}

    public void linkChannelWithSession(Channel c,UdpSimpleSession s) {};
}
